@echo -off
cls 0 15
echo " "
echo " *******************************************************************"
echo " *                 Kontron - Product Center Boards                 *"
echo " *                    EC Firmware Update script                    *"
echo " *******************************************************************"

# define variables for applications
set -v KoFind ".\EFI\Findstr.efi" > nul
set -v AFU ".\Afu\AfuEfix64.efi" > nul
set -v AFUParams "^^/ECR ^/RLC:F" > nul
set -v FWUpdLcl ".\FWUpdLcl\FWUpdLcl.efi" > nul

# clean up files and variables
rm smbios.out > nul
rm afu.out > nul
set -d FlashFile > nul
set -d product > nul
set -d FindStr > nul

# Read SMBIOS type 2 (Baseboard) to get the running board product type
smbiosview -t 2 >a smbios.out
%KoFind% /M /DK "ProductName*K*-??" smbios.out > nul
set -v product %FindStr% > nul
set -d FindStr > nul

# Read SMBIOS type 0 (BIOS) to get the running BIOS version
smbiosview -t 0 >>a smbios.out
%KoFind% /M /I /D".?? " /E" for " "BiosVersion*R?.*.*" smbios.out > nul
set -v BIOSV %FindStr% > nul
set -d FindStr > nul

# Read SMBIOS type 0 (BIOS) to get the running EC Firmware version
%KoFind% /M /D" " "EmbeddedControllerFirmwareMajorRelease:*" smbios.out > nul
set -v ECFW %FindStr% > nul
set -d FindStr > nul
%KoFind% /M /D" " "EmbeddedControllerFirmwareMinorRelease:*" smbios.out > nul
set -v ECFW %ECFW%.%FindStr% > nul
set -d FindStr > nul

# If this script was called with an argument, assume it's the BIOS flash filename
# Continue without searching for a suitable file
set -v FlashFile "0.0.0" > nul
if "%1" neq "" then
  set -v FlashFile %1 > nul
endif
if %FlashFile% eq "0.0.0" then
  # Search for suitable flash file in script directory, with highest revision
  for %f in K%product%*.bin
    if exists "%f" then
      if /i /s "%f" gt "%FlashFile%" then
        set -v FlashFile "%f" > nul
      endif
    endif
  endfor
endif

# Check if suitable flash BIN file was found
if "%FlashFile%" eq "0.0.0" then
  goto ERR_FILE
endif

# Make filename relative
%KoFind% /M /I /D"%cwd%\" "*" /S"%FlashFile%" > nul
set -v FlashFile %FindStr%

# BIOS version from Filename
%KoFind% /M /I /D"K%product%." "R?.*.*" /E".BIN" /S"%FlashFile%" > nul
set -v BIOS_TARGET_VERSION %FindStr% > nul
set -d FindStr > nul

# Check ROM ID from flashfile and compare it with product name of the board
%AFU% %FlashFile% /U >a afu.out
%KoFind% /M /D" K" "ROM File ROM ID = K*-??" afu.out > nul
if %product% neq %FindStr% then
  set -d FindStr > nul
  goto ERR_ROMID
endif

# Check if ME is inactive for EC Firmware flash
%FWUpdLcl% -fwver > nul
set -v FWUpdLclERR %lasterror%
if %FWUpdLclERR% eq "0x1C0B" then
  set -v EC_HOWTO_SHOWN true
else
  set -d EC_HOWTO_SHOWN > nul
endif

#Show message to user
:EXECUTE
echo " "
echo " > EC Firmware for motherboard  : K%product%"
echo " > BIOS version installed       : %BIOSV%"
echo " > EC Firmware installed        : %ECFW%"
echo " > EC Firmware flash file name  : %FlashFile%"
echo " "
if "%EC_HOWTO_SHOWN%" eq "" then
  goto REBOOT_TO_BIOS_SETUP
endif
echo " *******************************************************************"
echo " *                                                                 *"
echo " *              Are you sure to update the EC Firmware?            *"
echo " *            Press 'q' to abort and any key to continue           *"
echo " *                                                                 *"
echo " *******************************************************************"
echo " "
pause > nul

:FLASH_START
%AFU% %FlashFile% %AFUParams%
set -v AFUERR %lasterror%
if %AFUERR% neq "0x0" then
  goto ERR_AFU
endif 

:FLASH_DONE
rm smbios.out > nul
rm afu.out > nul
set -d FlashFile > nul
set -d product > nul
set -d FindStr > nul
set -d ME_flashed > nul

# Automatic restart...
echo " "
echo " *******************************************************************"
echo " *                                                                 *"
echo " *  If the message 'ECR is locked, skip updating' appeared         *"
echo " *  use 'BIOS Service Updater' prior to EC update script.          *"
echo " *                                                                 *"
echo " *  A power cycle is necessary.                                    *"
echo " *          1. Wait until the system switched off                  *"
echo " *          2. Remove AC cable                                     *"
echo " *          3. Wait until all voltage rails are discharged         *"
echo " *          4. Re-connect AC cable an switch system on             *"
echo " *                                                                 *"
echo " *  Press 'q' to abort or any key to shutdown now.                 *"
echo " *                                                                 *"
echo " *******************************************************************"
pause > nul
reset -s
goto EOF

:REBOOT_TO_BIOS_SETUP
cls 0 3
echo " "
echo " *******************************************************************"
echo " *                                                                 *"
echo " *        For updating the EC Firmware it is necessary             *"
echo " *         to change a BIOS setting manually before:               *"
echo " *                                                                 *"
echo " *  At tab ^"Security^" set ^"EC Firmware Update^" to ^"Enabled^"        *"
echo " *  and leave BIOS setup with F4 (Save Changes and Reset).         *"
echo " *                                                                 *"
echo " *  Please note: The setting will only remain valid for one boot!  *"
echo " *                                                                 *"
echo " *  Press 'q' to abort or                                          *"
echo " *  any key to continue rebooting to BIOS Setup                    *"
echo " *                                                                 *"
echo " *******************************************************************"
echo " "
pause > nul
reset -fwui
goto EOF

:ERR_FILE
echo " "
echo " *******************************************************************"
echo " *   ERROR: Flash file and board does not match!                   *"
echo " *******************************************************************"
echo " "
goto EOF      

:ERR_ROMID
echo " "
echo " *******************************************************************"
echo " *   ERROR: ROM ID from Flashfile does not match to system!        *"
echo " *******************************************************************"
echo " "
goto EOF 

:ERR_AFU
echo " "
echo " *******************************************************************"
echo " *   ERROR: AFU flash tool reported an error.                      *"
echo " *          Please check and try again.                            *"
echo " *******************************************************************"
echo " "
echo " Returncode: %AFUERR%"
echo " "
goto EOF   

:EOF
