@echo off
cls 
SetLocal EnableExtensions EnableDelayedExpansion

REM Check for administratior privileges. If not - please start as admin!
net session >nul 2>&1
IF NOT %ERRORLEVEL%==0 (
  ECHO You must right-click and select
  ECHO "RUN AS ADMINISTRATOR"  to run this batch. Exiting... 
  ECHO.
  PAUSE
  EXIT /D
)
  
echo.
echo  ###################################################################
echo  #                 Kontron - Product Center Boards                 #
echo  #                    EC Firmware Update script                    #
echo  ###################################################################

pushd "%~dp0"
set AFU=Afu\AFUWINx64.exe
set AFUParams=/ECR /RLC:F
set MeDevId=54E0

set FlashFile=%~n1%~x1

REM Set variables which have to remain after reboots
if not exist  _statusvars.cmd (
  REM if file does not exist write header to it
  echo. > _statusvars.cmd
  echo REM This file is needed for the update process. Please do not remove manually. >> _statusvars.cmd
  echo @echo off >> _statusvars.cmd
) else (
  REM Execute file
  call _statusvars.cmd > nul
)

REM Read SMBIOS type 2 (Baseboard) to get the running board product type
FOR /F "skip=2 tokens=2 delims=," %%A IN ('wmic baseboard get product /FORMAT:csv') do (set "product=%%A")

REM Read SMBIOS type 0 (BIOS) to get the running BIOS version
FOR /F "skip=2 tokens=2 delims= " %%A IN ('wmic bios get SmBiosBiosVersion /FORMAT:csv') do (set "BIOSV=%%A")

REM Read SMBIOS type 0 (BIOS) to get the running EC Firmware version
FOR /F "tokens=2 delims=," %%A IN ('wmic bios get EmbeddedControllerMajorVersion /FORMAT:csv') do (set "ECFW=%%A")
FOR /F "tokens=2 delims=," %%A IN ('wmic bios get EmbeddedControllerMinorVersion /FORMAT:csv') do (set "ECFW=%ECFW%.%%A")

REM If this script was called with an argument, assume it's the BIOS flash filename
REM Continue without searching for a suitable file
IF "%FlashFile%"=="" (
  REM Search for a suitable one in the script directory, with highest revision
  FOR %%F IN (%product%*.bin) do (
          if /i %%~nxF GTR !FlashFile! (
        set FlashFile=%%~nxF
          )
  )
  REM Check if suitable flash BIN file was found
  IF "!FlashFile!" == "" goto ERR_FILE
)

REM BIOS version from Filename 
call set BIOS_TARGET_VERSION=%%FlashFile:%product%.=%%
set BIOS_TARGET_VERSION=%BIOS_TARGET_VERSION:.BIN=%

REM Determine status of ME Interface
wmic path win32_pnpentity where "deviceid like '%%DEV_%%%MeDevId%%%%%'" get status 2> nul | findstr /i "Error OK" > nul
if not %ERRORLEVEL% == 0 (
  REM ME is deactivated
  set EC_HOWTO_SHOWN=true
)

REM Check ROM ID from flashfile and compare it with product name of the board
%AFU% %FlashFile% /U | findstr /I "%product%" > nul
if not %ERRORLEVEL% == 0 goto ERR_ROMID

REM Show message to user
:EXECUTE
echo.
echo  o EC Firmware for motherboard  : %product%
echo  o BIOS version installed       : %BIOSV%
echo  o EC Firmware installed        : %ECFW%
echo  o EC Firmware File             : %FlashFile%
echo.
if not "%EC_HOWTO_SHOWN%" == "true" (
  goto REBOOT_TO_BIOS_SETUP
)
echo  ###################################################################
echo  #                                                                 #
echo  #              Are you sure to update the EC Firmware?            #
echo  #                          ^<y^>es ^| ^<n^>o                           #
echo  #                                                                 #
echo  #      ATTENTION: System is blocked during BIOS update!           #
echo  #                 To prevent data loss it's recommended to        #
echo  #                 save and close all open documents.              #
echo  #                                                                 #
echo  ###################################################################
echo.
choice /C yn /N > nul
IF ERRORLEVEL 2 GOTO EOF
IF ERRORLEVEL 1 GOTO FLASH_START

:FLASH_START
set EC_HOWTO_SHOWN=
%AFU% %FlashFile% %AFUParams%
set AFUERR=%ERRORLEVEL%
if NOT %AFUERR%==0 (
  goto ERR_AFU
)

:FLASH_DONE
del /f _statusvars.cmd > nul

REM Automatic restart...
echo.
echo  ###################################################################
echo  #                                                                 #
echo  #  If the message 'ECR is locked, skip updating' appeared         #
echo  #  use 'BIOS Service Updater' prior to EC update script.          #
echo  #                                                                 #
echo  #  A power cycle is necessary.                                    #
echo  #          1. Wait until the system switched off                  #
echo  #          2. Remove AC cable                                     #
echo  #          3. Wait until all voltage rails are discharged         #
echo  #          4. Re-connect AC cable an switch system on             #
echo  #                                                                 #
echo  #  Do you want to shutdown now? ^<y^>es ^| ^<n^>o                      #
echo  #                                                                 #
echo  ###################################################################
echo.
choice /C yn /N > nul
IF ERRORLEVEL 2 GOTO EOF
IF ERRORLEVEL 1 shutdown -s -f -t 0

:REBOOT_TO_BIOS_SETUP
cls && color B
echo.
echo  ###################################################################
echo  #                                                                 #
echo  #        For updating the EC Firmware it is necessary             #
echo  #         to change a BIOS setting manually before:               #
echo  #                                                                 #
echo  #  At tab "Security" set "EC Firmware Update" to "Enabled"        #
echo  #  and leave BIOS setup with F4 (Save Changes and Reset).         #
echo  #                                                                 #
echo  #  Please note: The setting will only remain valid for one boot!  #
echo  #                                                                 #
echo  #  Do you want to reboot now to BIOS Setup? ^<y^>es ^| ^<n^>o          #
echo  #                                                                 #
echo  ###################################################################
echo.
choice /C yn /N > nul
IF ERRORLEVEL 2 GOTO EOF
IF ERRORLEVEL 1 (
  set EC_HOWTO_SHOWN=true
  REM The shutdown command has to be executed twice due to an error return after the first call!
  shutdown -r -fw -f -t 0 & shutdown -r -fw -f -t 0
  goto EOF
)

:ERR_FILE
echo. 
echo  ###################################################################
echo  #   ERROR: Flash file and board does not match^^!                   #
echo  ###################################################################
echo. 
goto EOF

:ERR_ROMID
echo.
echo  ###################################################################
echo  #   ERROR: ROM ID from Flashfile does not match to system^^!        #
echo  ###################################################################
echo.
goto EOF 

:ERR_AFU
echo. 
echo  ###################################################################
echo  #   ERROR: AFU flash tool reported an error.                      #
echo  #          Please check and try again                             #
echo  ###################################################################
echo. 
echo  Returncode: %AFUERR%
echo.
goto EOF


:EOF
pause
color 7
